#!/usr/bin/env python2.5

# Viscosity DNS Support Script
# http://www.viscosityvpn.com

import os, re, sys
from subprocess import *

nameservers = []
search_domains = []

foreign_option_vars = [k for k in os.environ.iterkeys()
                       if k.startswith('foreign_option_')]
foreign_option_vars.sort()
dhcp_options = [os.environ[k].split(None, 2)[1:] for k in foreign_option_vars
                if os.environ[k].startswith('dhcp-option ')]
for option, value in dhcp_options:
    if option == 'DOMAIN':
        search_domains.append(value)
    elif option == 'DNS':
        nameservers.append(value)

going_up = False

if not search_domains:
	search_domains.append("openvpn")
if not nameservers:
	sys.exit(0)

try:
    tun_dev, tun_mtu, link_mtu, ifconfig_local_ip, ifconfig_remote_ip, \
      command = sys.argv[1:]
except:
	sys.exit(0)

from SystemConfiguration2 import *
SC_IPV4_PATH, SC_DNS_PATH = ['/'.join((kSCDynamicStoreDomainState,
				   kSCCompNetwork, kSCCompService,
				   'net.openvpn.vpn-' + tun_dev, ent))
			 for ent in (kSCEntNetIPv4, kSCEntNetDNS)]
store = UNSystemConfigurationDynamicStore.alloc().initWithName_('OpenVPN')
if going_up:
	ipv4 = {kSCPropNetIPv4Addresses: [ifconfig_local_ip],
		kSCPropNetIPv4DestAddresses: [ifconfig_remote_ip],
		kSCPropInterfaceName: tun_dev}
	kSCPropSupplementalMatchDomains = 'SupplementalMatchDomains'
	dns = {kSCPropNetDNSServerAddresses: nameservers,
		   kSCPropSupplementalMatchDomains: search_domains}
	store.setValue_forKey_(ipv4, SC_IPV4_PATH)
	store.setValue_forKey_(dns, SC_DNS_PATH)
else:
	store.removeValueForKey_(SC_IPV4_PATH)
	store.removeValueForKey_(SC_DNS_PATH)
	
resolv_file_path = '/etc/resolv.conf'
if os.path.exists('/var/run/resolv.conf'):
	resolv_file_path = '/var/run/resolv.conf'

resolv_file = file(resolv_file_path, 'r+')
resolv_lines = resolv_file.readlines()

START_SKIP = '# begin %s OpenVPN tunnel modifications' % tun_dev
END_SKIP = '# end %s OpenVPN tunnel modifications' % tun_dev

in_skip = False

resolv_file.seek(0)
resolv_file.truncate()
for line in resolv_lines:
	if in_skip:
		if re.match(END_SKIP, line):
			in_skip = False
		continue
	else:
		if re.match(START_SKIP, line):
			in_skip = True
			continue
	if line == '\n': continue
	line = re.sub('# ', '', line)
	resolv_file.write(line)

resolv_file.close()

cmd = 'dscacheutil -flushcache'
p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
p.stdout.read()

