#!/usr/bin/env python2.5

# Viscosity DNS Support Script
# http://www.viscosityvpn.com


import os, re, sys, time, platform
from subprocess import *

# Update path for running instance to make sure we can find the programs
os.environ["PATH"] = '/bin:/sbin:/usr/sbin:/usr/bin'

def networkReset():
	byPassList = ['lo', 'vmnet', 'tun', 'tap']
	blockList = []

	# Get airport status (workaround for Mac OS 10.5.6+ bug/change)
	osVersion = platform.mac_ver()[0].rsplit('.', 1)
	if (osVersion[0] == '10.5' and int(osVersion[1]) >= 6):
		# Get Airport interface
		p = Popen("networksetup -listallhardwareports", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
		airportFlag = False
		data = p.stdout.read().split('\n')
		for line in data:
			if airportFlag:
				airportInterface = line.split()[1]
				blockList.append(airportInterface)
				break
			if line.find("Hardware Port: AirPort") > -1:
				airportFlag = True
		p.stdout.close()

		p = Popen("networksetup -getairportpower", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
		if p.stdout.read().find("AirPort Power: On") > -1:
			airportOn = True
			cmd = "networksetup -setairportpower off"
			p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
			p.stdout.close()
		else:
			airportOn = False
		p.stdout.close()
		
		

	time.sleep(1)

	p = Popen("ifconfig |grep \<UP", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
	output = p.stdout.readlines()
	for line in output:
		interface = line.split(":", 1)[0]
		if interface.strip("0123456789") not in byPassList and interface not in blockList:
			os.system('/sbin/ifconfig %s down' % interface)

	time.sleep(1)

	# Restore airport status
	if (osVersion[0] == '10.5' and int(osVersion[1]) >= 6):
		if airportOn:
			cmd = "networksetup -setairportpower on"
		else:
			cmd = "networksetup -setairportpower off"
		p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
		p.stdout.close()

	time.sleep(4)

	for line in output:
		interface = line.split(":", 1)[0]
		if interface.strip("0123456789") not in byPassList and interface not in blockList:
			os.system('/sbin/ifconfig %s up' % interface)
	p.stdout.close()

networkReset()
nameservers = []
search_domains = []

foreign_option_vars = [k for k in os.environ.iterkeys()
                       if k.startswith('foreign_option_')]
foreign_option_vars.sort()
dhcp_options = [os.environ[k].split(None, 2)[1:] for k in foreign_option_vars
                if os.environ[k].startswith('dhcp-option ')]
for option, value in dhcp_options:
    if option == 'DOMAIN':
        search_domains.append(value)
    elif option == 'DNS':
        nameservers.append(value)

going_up = False

if not search_domains:
	search_domains.append("openvpn")
if not nameservers:
	networkReset()
	sys.exit(0)

try:
    tun_dev, tun_mtu, link_mtu, ifconfig_local_ip, ifconfig_remote_ip, command = sys.argv[1:]
except:
	networkReset()
	sys.exit(0)

try:
	networkReset()
	from SystemConfiguration2 import *
	SC_IPV4_PATH, SC_DNS_PATH = ['/'.join((kSCDynamicStoreDomainState,
					   kSCCompNetwork, kSCCompService,
					   'net.openvpn.vpn-' + tun_dev, ent))
				 for ent in (kSCEntNetIPv4, kSCEntNetDNS)]
	store = UNSystemConfigurationDynamicStore.alloc().initWithName_('OpenVPN')
	if going_up:
		ipv4 = {kSCPropNetIPv4Addresses: [ifconfig_local_ip],
			kSCPropNetIPv4DestAddresses: [ifconfig_remote_ip],
			kSCPropInterfaceName: tun_dev}
		kSCPropSupplementalMatchDomains = 'SupplementalMatchDomains'
		dns = {kSCPropNetDNSServerAddresses: nameservers,
			   kSCPropSupplementalMatchDomains: search_domains}
		store.setValue_forKey_(ipv4, SC_IPV4_PATH)
		store.setValue_forKey_(dns, SC_DNS_PATH)
	else:
		store.removeValueForKey_(SC_IPV4_PATH)
		store.removeValueForKey_(SC_DNS_PATH)
		
	resolv_file_path = '/etc/resolv.conf'
	if os.path.exists('/var/run/resolv.conf'):
		resolv_file_path = '/var/run/resolv.conf'

	resolv_file = file(resolv_file_path, 'r+')
	resolv_lines = resolv_file.readlines()

	START_SKIP = '# begin %s OpenVPN tunnel modifications' % tun_dev
	END_SKIP = '# end %s OpenVPN tunnel modifications' % tun_dev
	search_domains = ['search'] + search_domains

	in_skip = False
	search_line = ' '.join(search_domains)

	resolv_file.seek(0)
	resolv_file.truncate()
	for line in resolv_lines:
		if in_skip:
			if re.match(END_SKIP, line):
				in_skip = False
			continue
		else:
			if re.match(START_SKIP, line):
				in_skip = True
				continue
		if line == '\n':
			continue
		if going_up:
			if re.match('nameserver ', line):
				resolv_file.write('# ')
			if re.match('search ', line):
				resolv_file.write('# ')
				search_line = ' '.join(search_domains + \
									   [domain for domain in line[:-1].split(' ')
										if domain not in search_domains])
		else:
			line = re.sub('# ', '', line)
		resolv_file.write(line)

	resolv_file.close()

	networkReset()
	
	cmd = 'dscacheutil -flushcache'
	p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
	p.stdout.read()
	
except:
	import traceback
	file = open('/Library/Logs/Viscosity-DNS.log', 'w')
	file.write(traceback.format_exc())
	file.close()