#!/usr/bin/python

# Viscosity DNS Support Script
# http://www.viscosityvpn.com

import os, sys, re, time, platform
from subprocess import *

# Update path for running instance to make sure we can find the programs
os.environ["PATH"] = '/bin:/sbin:/usr/sbin:/usr/bin'

# Get command line args
try:
    tun_dev, tun_mtu, link_mtu = sys.argv[1:4]
except:
	sys.exit(0)
	
# Get primary network interface
cmd = '''(scutil | grep PrimaryService | sed -e 's/.*PrimaryService : //')<< EOF
open
get State:/Network/Global/IPv4
d.show
quit
EOF'''
p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
networkInt = p.stdout.read()[:-1]

if os.path.isfile('/tmp/openvpn_dhcprestore'):
	os.remove('/tmp/openvpn_dhcprestore')

# Restore old settings
if os.path.isfile('/tmp/openvpn_domain_' + networkInt):
	tempfile = open('/tmp/openvpn_domain_' + networkInt, 'r')
	oldDomain = tempfile.read()
	tempfile.close()
	os.remove('/tmp/openvpn_domain_' + networkInt)
else:
	oldDomain = ''

if os.path.isfile('/tmp/openvpn_dns_' + networkInt):
	tempfile = open('/tmp/openvpn_dns_' + networkInt, 'r')
	oldDNS = tempfile.read()
	tempfile.close()
	os.remove('/tmp/openvpn_dns_' + networkInt)
		
	# Remove DNS settings
	cmd = '''scutil << EOF
	open
	d.init
	d.add ServerAddresses * %s
	d.add DomainName %s
	set State:/Network/Service/%s/DNS
	quit
	EOF''' % (oldDNS, oldDomain, networkInt)
	p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
	p.stdout.read() # Wait for EOF
		
	resolv_file_path = '/etc/resolv.conf'
	if os.path.exists('/var/run/resolv.conf'):
		resolv_file_path = '/var/run/resolv.conf'

	resolv_file = file(resolv_file_path, 'r+')
	resolv_lines = resolv_file.readlines()

	START_SKIP = '# begin %s OpenVPN tunnel modifications' % tun_dev
	END_SKIP = '# end %s OpenVPN tunnel modifications' % tun_dev

	in_skip = False

	resolv_file.seek(0)
	resolv_file.truncate()
	for line in resolv_lines:
		if in_skip:
			if re.match(END_SKIP, line):
				in_skip = False
			continue
		else:
			if re.match(START_SKIP, line):
				in_skip = True
				continue
		if line == '\n': continue
		line = re.sub('# ', '', line)
		resolv_file.write(line)

	resolv_file.close()

if os.path.isfile('/tmp/openvpn_staticdns_' + networkInt):
	tempfile = open('/tmp/openvpn_staticdns_' + networkInt, 'r')
	staticDNS = tempfile.read()
	tempfile.close()
	os.remove('/tmp/openvpn_staticdns_' + networkInt)
		
	cmd = '''scutil << EOF
	open
	d.init
	d.add ServerAddresses * %s
	set Setup:/Network/Service/%s/DNS
	quit
	EOF''' % (staticDNS, networkInt)
	p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
	p.stdout.read() # Wait for EOF

byPassList = ['lo', 'vmnet', 'tun', 'tap']
blockList = []

# Get airport status (workaround for Mac OS 10.5.6+ bug/change)
osVersion = platform.mac_ver()[0].rsplit('.', 1)
if (osVersion[0] == '10.5' and int(osVersion[1]) >= 6):
	# Get Airport interface
	p = Popen("networksetup -listallhardwareports", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
	airportFlag = False
	data = p.stdout.read().split('\n')
	for line in data:
		if airportFlag:
			airportInterface = line.split()[1]
			blockList.append(airportInterface)
			break
		if line.find("Hardware Port: AirPort") > -1:
			airportFlag = True
	p.stdout.close()

	p = Popen("networksetup -getairportpower", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
	if p.stdout.read().find("AirPort Power: On") > -1:
		airportOn = True
		cmd = "networksetup -setairportpower off"
		p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
		p.stdout.close()
	else:
		airportOn = False
	p.stdout.close()
	
	

time.sleep(1)

p = Popen("ifconfig |grep \<UP", shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
output = p.stdout.readlines()
for line in output:
	interface = line.split(":", 1)[0]
	if interface.strip("0123456789") not in byPassList and interface not in blockList:
		os.system('/sbin/ifconfig %s down' % interface)

time.sleep(1)

# Restore airport status
if (osVersion[0] == '10.5' and int(osVersion[1]) >= 6):
	if airportOn:
		cmd = "networksetup -setairportpower on"
	else:
		cmd = "networksetup -setairportpower off"
	p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, close_fds=True)
	p.stdout.close()

time.sleep(4)

for line in output:
	interface = line.split(":", 1)[0]
	if interface.strip("0123456789") not in byPassList and interface not in blockList:
		os.system('/sbin/ifconfig %s up' % interface)
p.stdout.close()

cmd = 'dscacheutil -flushcache'
p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
p.stdout.read()
